@@ -37,10 +37,13 @@ def _parse_args(args_arr):
     parser.add_argument(
         '--altitude', dest='altitude', metavar='', nargs=2, type=str,
         default=[0, 'kft'],
-        help="altitude and lenght unit, default='0 kft'")
+        help="altitude and length unit, default='0 kft'")
     parser.add_argument(
-        '--Mach', dest='M', metavar='', nargs=None, type=float,
+        '-M', '--Mach', dest='M', metavar='', nargs=None, type=float,
         default=None, help="Mach number, e.g. '0.5'")
+    parser.add_argument(
+        '--mach-number', dest='M', metavar='', nargs=None, type=float,
+        default=None, help=argparse.SUPPRESS)
     parser.add_argument(
         '--TAS', dest='TAS', metavar='', nargs=2, type=str, default=None,
         help="true airspeed and speed unit, e.g. '150 knots'")
@@ -51,8 +54,26 @@ def _parse_args(args_arr):
         '--EAS', dest='EAS', metavar='', nargs=2, type=str, default=None,
         help="equivalent airspeed and speed unit, e.g. '150 knots'")
     parser.add_argument(
-        '--length', dest='length_scale', metavar='', nargs=2, type=str,
-        default=None, help="length scale, e.g. '10 ft'")
+        '--KTAS', dest='KTAS', metavar='', nargs=None, type=float,
+        default=None, help=argparse.SUPPRESS)
+    parser.add_argument(
+        '--KCAS', dest='KCAS', metavar='', nargs=None, type=float,
+        default=None, help=argparse.SUPPRESS)
+    parser.add_argument(
+        '--KEAS', dest='KEAS', metavar='', nargs=None, type=float,
+        default=None, help=argparse.SUPPRESS)
+    parser.add_argument(
+        '-L', '--length-scale', dest='length_scale', metavar='', nargs=2,
+        type=str, default=None, help="length scale, e.g. '10 ft'")
+    parser.add_argument(
+        '--ell', dest='length_scale', metavar='', nargs=2,
+        type=str, default=None, help=argparse.SUPPRESS)
+    parser.add_argument(
+        '--unit-system', dest='unit_system', metavar='', nargs=None,
+        type=str, default="", help=f"Unit system, i.e. {dir(unit.sys)}")
+    parser.add_argument(
+        '--abbreviated-output', dest='abbreviated_output', default=False,
+        action='store_true', help="display abbreviated output")
     parser.add_argument(  # hidden option to turn off pretty print
         '--no-pretty-print', dest='no_pretty_print', default=False,
         action='store_true', help=argparse.SUPPRESS)
@@ -84,15 +105,20 @@ def main():
     TAS_ = None if args.TAS is None else _dimension(args.TAS)
     CAS_ = None if args.CAS is None else _dimension(args.CAS)
     EAS_ = None if args.EAS is None else _dimension(args.EAS)
+    TAS_ = TAS_ if args.KTAS is None else args.KTAS*unit('knots')
+    CAS_ = CAS_ if args.KCAS is None else args.KCAS*unit('knots')
+    EAS_ = EAS_ if args.KEAS is None else args.KEAS*unit('knots')
     L_ = None if args.length_scale is None else _dimension(args.length_scale)
-    fc = FlightCondition(h=h_, M=M_, TAS=TAS_, CAS=CAS_, EAS=EAS_, L=L_)
+    fc = FlightCondition(h=h_, M=M_, TAS=TAS_, CAS=CAS_, EAS=EAS_, L=L_,
+                         unit_system=args.unit_system)
 
     # Print output but catch common unicode exception
+    full_output = not args.abbreviated_output
     try:
-        print(fc.tostring(full_output=True,
+        print(fc.tostring(full_output=full_output,
               pretty_print=(not args.no_pretty_print)))
     except UnicodeEncodeError:
-        print(fc.tostring(full_output=True, pretty_print=False))
+        print(fc.tostring(full_output=full_output, pretty_print=False))
 
 
 if __name__ == '__main__':
