@@ -10,9 +10,11 @@ Email: matt.c.jones.aoe@gmail.com
 :license: MIT License, see LICENSE for more details.
 """
 
+import warnings
+
 from functools import wraps
-from numpy import atleast_1d, exp, ndarray, pi, shape, size, str_, sqrt,\
-    zeros_like
+
+import numpy as np
 
 from flightcondition.constants import PhysicalConstants as Phys
 from flightcondition.constants import AtmosphereConstants as Atmo
@@ -33,8 +35,8 @@ def _len1array_to_scalar(func):
     @wraps(func)
     def wrapper(*args, **kwargs):
         output = func(*args, **kwargs)
-        if isinstance(output.magnitude, ndarray):
-            if size(output) == 1:
+        if isinstance(output.magnitude, np.ndarray):
+            if np.size(output) == 1:
                 return output[0]
         return output
     return wrapper
@@ -54,7 +56,7 @@ def _formatarr(unitstr):
         def wrapper(*args, **kwargs):
             arr = func(*args, **kwargs)
             arr *= unit(unitstr)
-            if size(arr) == 1:
+            if np.size(arr) == 1:
                 arr = arr[0]
             return arr
         return wrapper
@@ -71,30 +73,65 @@ def _property_decorators(func):
         @_len1array_to_scalar
     """
     return property(to_base_units_wrapper(_len1array_to_scalar(func)))
-    # return _len1array_to_scalar(to_base_units_wrapper(property(func)))
 
 
-class AccessByName():
-    """Nested class to reference quantities by their full name. """
+class AliasAttributes():
+    """Nested alias class to reference quantities by prescribed aliases. """
 
-    def _populate_data(self, varsobj, varnames_dict):
+    def __init__(self, varsobj_arr, varnames_dict_arr):
         """Populate full names and link to variable
         Args:
-            varnames_dict: Dictionary that maps variables to their longer
-                names
-            varsobj: Object that holds all of the variables
+            varsobj (list): List of objects that holds all of the variables
+            varnames_dict (list): List of dictionaries that maps variables to
+                their alias names
         """
-        for var, varname in varnames_dict.items():
-            setattr(self, varname, getattr(varsobj, var))
+        # Avoid infinite loop when setting properties
+        super().__setattr__("varsobj_arr", varsobj_arr)
+        super().__setattr__("varnames_dict_arr", varnames_dict_arr)
+
+    def __dir__(self):
+        """Add tab completion for alias names. """
+        # Avoid infinite loop when loading varsobj and varnames_dict
+        varnames_dict_arr = super().__getattribute__("varnames_dict_arr")
+        varnames_arr = []
+        for varnames_dict in varnames_dict_arr:
+            for varname in varnames_dict.values():
+                varnames_arr.append(varname)
+        return varnames_arr
+
+    def __getattribute__(self, attr):
+        """Get referenced quantity
+
+        Args:
+            attr (str): attribute name
+        """
+        # Avoid infinite loop when loading varsobj and varnames_dict
+        varsobj_arr = super().__getattribute__("varsobj_arr")
+        varnames_dict_arr = super().__getattribute__("varnames_dict_arr")
+        for varsobj, varnames_dict in zip(varsobj_arr, varnames_dict_arr):
+            for var, varname in varnames_dict.items():
+                if attr == varname:
+                    return getattr(varsobj, var)
+
+    def __setattr__(self, attr, attrval):
+        """Set referenced quantity
+
+        Args:
+            attr (str): attribute name
+            attrval (quantity): attribute value
+        """
+        # Avoid infinite loop when loading varsobj and varnames_dict
+        varsobj_arr = super().__getattribute__("varsobj_arr")
+        varnames_dict_arr = super().__getattribute__("varnames_dict_arr")
+        for varsobj, varnames_dict in zip(varsobj_arr, varnames_dict_arr):
+            for var, varname in varnames_dict.items():
+                if attr == varname:
+                    return setattr(varsobj, var, attrval)
 
 
 class DimensionalData:
     """Parent class to hold dimensional data"""
 
-    def __init__(self):
-        """Initialize object. """
-        self.byname = AccessByName()
-
     def __str__(self):
         """Output string when object is printed.
 
@@ -111,6 +148,22 @@ class DimensionalData:
         """
         return self.tostring(full_output=False)
 
+    @staticmethod
+    def _arg_from_alias(alias_list, kwargs_dict):
+        """Determine argument from hidden alias list
+
+        Args:
+            alias_list (dict): list of possible argument aliases
+
+        Returns:
+            unit.Quantity: argument
+        """
+        for alias in alias_list:
+            if alias in kwargs_dict.keys():
+                return kwargs_dict[alias]
+        else:  # no break
+            return None
+
 
 class Atmosphere(DimensionalData):
     """Compute quantities from International Civil Aviation Organization (ICAO)
@@ -127,7 +180,7 @@ class Atmosphere(DimensionalData):
         #print(f"\n{atm}")
 
         # Uncomment to print while specifying abbreviated output in US units:
-        #print(f"\n{atm.tostring(full_output=False, US_units=True)}")
+        #print(f"\n{atm.tostring(full_output=False, unit_system='US')}")
 
         # See also the linspace() function from numpy, e.g.
         # h = linspace(0, 81.0, 82) * unit('km')
@@ -167,13 +220,13 @@ class Atmosphere(DimensionalData):
                 H_arr (length): Geopotential altitude
 
             """
-            H_arr = atleast_1d(H_arr)
+            H_arr = np.atleast_1d(H_arr)
 
-            self._layer_name = [""]*size(H_arr)
-            self._H_base = zeros_like(H_arr) * unit('m')
-            self._T_base = zeros_like(H_arr) * unit('K')
-            self._T_grad = zeros_like(H_arr) * unit('K/m')
-            self._p_base = zeros_like(H_arr) * unit('Pa')
+            self._layer_name = [""]*np.size(H_arr)
+            self._H_base = np.zeros_like(H_arr) * unit('m')
+            self._T_base = np.zeros_like(H_arr) * unit('K')
+            self._T_grad = np.zeros_like(H_arr) * unit('K/m')
+            self._p_base = np.zeros_like(H_arr) * unit('Pa')
 
             for idx, H in enumerate(H_arr):
                 jdx = __class__._layer_idx(H)
@@ -204,7 +257,7 @@ class Atmosphere(DimensionalData):
         @property
         def name(self):
             """Layer name """
-            if size(self._layer_name) == 1:
+            if np.size(self._layer_name) == 1:
                 layername = self._layer_name[0]
             else:
                 layername = self._layer_name
@@ -230,89 +283,63 @@ class Atmosphere(DimensionalData):
             """Layer base pressure :math:`p_{base}` """
             return self._p_base
 
-    def __init__(self, h=0*unit('kft')):
+    def __init__(self, h=None, unit_system="", **kwargs):
         """Input geometric altitude - object contains the corresponding
         atmospheric quantities.
 
         Args:
-            h (length): Geometric altitude
+            h (length): Geometric altitude - aliases are 'alt', 'altitude'
+            unit_system (str): Set to 'US' for US units or 'SI' for SI
         """
         # Compute altitude bounds
         self._H_min = Atmo.H_base[0]
         self._H_max = Atmo.H_base[-1]
-        self._h_min = __class__.h_from_H(self._H_min)
-        self._h_max = __class__.h_from_H(self._H_max)
+        self._h_min = __class__._h_from_H(self._H_min)
+        self._h_max = __class__._h_from_H(self._H_max)
 
         # Process altitude input
-        self._h = self._process_input_altitude(h)
-        self._H = self.H_from_h(self.h)
-        self.layer = __class__.Layer(self.H)
+        # Check for hidden aliases
+        h_aliases = ['alt', 'altitude']
+        if h is None:
+            h = __class__._arg_from_alias(h_aliases, kwargs)
 
-        # Allow access to variables using full names
-        super().__init__()
-        self.byname._populate_data(varsobj=self, varnames_dict=self.varnames)
+        # Default to 0 kft
+        if h is None:
+            h = 0 * unit('ft')
+        self.h = h
 
-    def _process_input_altitude(self, alt):
-        """Check that input is of type Quantity from pint package. Check that
-        input is length dimension.  Check bounds.  Format as array even if
-        scalar input.
+        # Process unit system
+        if unit_system not in dir(unit.sys):  # check if usable system
+            if check_US_length_units(h):
+                self.unit_system = 'US'
+            else:
+                self.unit_system = 'SI'
+        else:
+            self.unit_system = unit_system
 
-        Args:
-            alt (length): Input scalar or array of altitudes
+        # Initialize access by full quantity name through .byname.<name>
+        self.byname = AliasAttributes(varsobj_arr=[self, ],
+                                      varnames_dict_arr=[__class__.varnames, ])
 
-        Returns:
-            length: Geometric altitude
-        """
-        tofloat = 1.0
-        h = atleast_1d(alt) * tofloat
-
-        check_dimensioned(h)
-        check_length_dimensioned(h)
-
-        if len(shape(h)) > 1:
-            raise TypeError("Input must be scalar or 1-D array.")
-
-        if (h < self._h_min).any() or (self._h_max < h).any():
-            raise ValueError(
-                f"Input altitude is out of bounds "
-                f"({self._h_min:.5g} < h < {self._h_max:.5g})"
-            )
-
-        self.US_units = check_US_length_units(h)
-        if self.US_units:
-            unit.default_system = 'US'  # 'US', 'imperial'
-
-        return h
-
-    @to_base_units_wrapper
-    def knudsen_number(self, ell):
-        """Compute the Knudsen number :math:`K_n`
-
-        Args:
-            ell (length): Length scale
-
-        Returns:
-            dimless: Knudsen number
-        """
-        Kn = self.MFP/ell
-        return Kn
-
-    def tostring(self, full_output=True, US_units=None, max_var_chars=0,
+    def tostring(self, full_output=True, unit_system=None, max_var_chars=0,
                  pretty_print=True):
         """String representation of data structure.
 
         Args:
             full_output (bool): Set to True for full output
-            US_units (bool): Set to True for US units and False for SI
+            unit_system (str): Set to 'US' for US units or 'SI' for SI
             max_var_chars (int): Maximum number of characters in unit string
             pretty_print (bool): Pretty print output
 
         Returns:
             str: String representation of class object
         """
-        US_units = self.US_units if US_units is None else US_units
         pp_ = '~P' if pretty_print else ''
-        if US_units:
+
+        if unit_system is not None:
+            self.unit_system = unit_system
+
+        if self.unit_system == 'US':
             h_str   = f"h       = {self.h.to('kft'):10.5g{pp_}}"
             H_str   = f"H       = {self.H.to('kft'):10.5g{pp_}}"
             p_str   = f"p       = {self.p.to('lbf/ft^2'):10.5g{pp_}}"
@@ -339,23 +366,23 @@ class Atmosphere(DimensionalData):
 
         # Insert longer variable name into output
         max_var_chars = max([
-            max([len(v) for v in self.varnames.values()]),
+            max([len(v) for v in __class__.varnames.values()]),
             max_var_chars
         ])
-        h_str   = f"{self.varnames['h']:{max_var_chars}s} {h_str}"
-        H_str   = f"{self.varnames['H']:{max_var_chars}s} {H_str}"
-        p_str   = f"{self.varnames['p']:{max_var_chars}s} {p_str}"
-        T_str   = f"{self.varnames['T']:{max_var_chars}s} {T_str}"
-        rho_str = f"{self.varnames['rho']:{max_var_chars}s} {rho_str}"
-        a_str   = f"{self.varnames['a']:{max_var_chars}s} {a_str}"
-        mu_str  = f"{self.varnames['mu']:{max_var_chars}s} {mu_str}"
-        nu_str  = f"{self.varnames['nu']:{max_var_chars}s} {nu_str}"
-        k_str   = f"{self.varnames['k']:{max_var_chars}s} {k_str}"
-        g_str   = f"{self.varnames['g']:{max_var_chars}s} {g_str}"
-        MFP_str = f"{self.varnames['MFP']:{max_var_chars}s} {MFP_str}"
+        h_str   = f"{__class__.varnames['h']:{max_var_chars}s} {h_str}"
+        H_str   = f"{__class__.varnames['H']:{max_var_chars}s} {H_str}"
+        p_str   = f"{__class__.varnames['p']:{max_var_chars}s} {p_str}"
+        T_str   = f"{__class__.varnames['T']:{max_var_chars}s} {T_str}"
+        rho_str = f"{__class__.varnames['rho']:{max_var_chars}s} {rho_str}"
+        a_str   = f"{__class__.varnames['a']:{max_var_chars}s} {a_str}"
+        mu_str  = f"{__class__.varnames['mu']:{max_var_chars}s} {mu_str}"
+        nu_str  = f"{__class__.varnames['nu']:{max_var_chars}s} {nu_str}"
+        k_str   = f"{__class__.varnames['k']:{max_var_chars}s} {k_str}"
+        g_str   = f"{__class__.varnames['g']:{max_var_chars}s} {g_str}"
+        MFP_str = f"{__class__.varnames['MFP']:{max_var_chars}s} {MFP_str}"
 
         if full_output:
-            if type(self.layer.name) is str_:  # singular string
+            if type(self.layer.name) is np.str_:  # singular string
                 trunc_layer_name = self.layer.name
             else:
                 trunc_layer_name = "[" + " ".join([
@@ -373,7 +400,7 @@ class Atmosphere(DimensionalData):
         return repr_str
 
     @staticmethod
-    def H_from_h(h):
+    def _H_from_h(h):
         """Convert geometric to geopotential altitude.
 
         :math:`H = \\frac{R_{earth} h}{R_{earth} + h}`
@@ -389,7 +416,7 @@ class Atmosphere(DimensionalData):
         return H
 
     @staticmethod
-    def h_from_H(H):
+    def _h_from_H(H):
         """Convert geopotential to geometric altitude.
 
         :math:`h = \\frac{R_{earth} H}{R_{earth} - H}`
@@ -404,11 +431,73 @@ class Atmosphere(DimensionalData):
         h = R_earth*H/(R_earth - H)
         return h
 
+    @property
+    def unit_system(self):
+        """Unit system to use: 'SI', 'US', etc.  Available unit systems given
+        by dir(unit.sys).
+
+        Returns:
+            str: Unit system
+        """
+        return self._unit_system
+
+    @unit_system.setter
+    def unit_system(self, unit_system):
+        """Unit system to use: 'SI', 'US', etc.  Available unit systems given
+        by dir(unit.sys).
+
+        Args:
+            unit_system (str): Unit system
+        """
+        if unit_system not in dir(unit.sys):
+            warnings.warn(f"'{unit_system} is not available. Try one of the "
+                          f"following: {dir(unit.sys)}")
+            return
+        else:
+            self._unit_system = unit_system
+            unit.default_system = unit_system
+
     @_property_decorators
     def h(self):
-        """Geometric altitude :math:`h` """
+        """Get geometric altitude :math:`h`
+
+        Returns:
+            length: Geometric altitude
+        """
         return self._h
 
+    @h.setter
+    def h(self, h):
+        """Set geometric altitude :math:`h`
+
+        Check that input is of type Quantity from pint package. Check that
+        input is length dimension.  Check bounds.  Format as array even if
+        scalar input.
+
+        Args:
+            h (length): Input scalar or array of altitudes
+        """
+        tofloat = 1.0
+        h = np.atleast_1d(h) * tofloat
+
+        check_dimensioned(h)
+        h = h.magnitude * unit(str(h.units))  # force local unit registry
+        check_length_dimensioned(h)
+
+        if len(np.shape(h)) > 1:
+            raise TypeError("Input must be scalar or 1-D array.")
+
+        if (h < self._h_min).any() or (self._h_max < h).any():
+            raise ValueError(
+                f"Input altitude is out of bounds "
+                f"({self._h_min:.5g} < h < {self._h_max:.5g})"
+            )
+
+        # Update quantities
+        self._h = h
+        self._H = self._H_from_h(self.h)
+        self.layer = __class__.Layer(self.H)
+
     @_property_decorators
     def H(self):
         """Geopotential altitude :math:`H` """
@@ -417,21 +506,21 @@ class Atmosphere(DimensionalData):
     @_property_decorators
     def p(self):
         """Air pressure :math:`p` """
-        H_base = atleast_1d(self.layer.H_base)
-        T_base = atleast_1d(self.layer.T_base)
-        T_grad = atleast_1d(self.layer.T_grad)
-        p_base = atleast_1d(self.layer.p_base)
+        H_base = np.atleast_1d(self.layer.H_base)
+        T_base = np.atleast_1d(self.layer.T_base)
+        T_grad = np.atleast_1d(self.layer.T_grad)
+        p_base = np.atleast_1d(self.layer.p_base)
 
-        H = atleast_1d(self.H)
-        T = atleast_1d(self.T)
+        H = np.atleast_1d(self.H)
+        T = np.atleast_1d(self.T)
         g_0 = Phys.g
         R_air = Phys.R_air
 
-        p = zeros_like(H) * unit('Pa')
+        p = np.zeros_like(H) * unit('Pa')
 
         # Pressure equation changes between T_grad == 0 and T_grad != 0
         s = T_grad == 0
-        p[s] = p_base[s]*exp((-g_0/(R_air*T[s]))*(H[s] - H_base[s]))
+        p[s] = p_base[s]*np.exp((-g_0/(R_air*T[s]))*(H[s] - H_base[s]))
 
         s = T_grad != 0
         p[s] = p_base[s]*(
@@ -443,9 +532,9 @@ class Atmosphere(DimensionalData):
     @_property_decorators
     def T(self):
         """Ambient air temperature :math:`T` """
-        T_grad = atleast_1d(self.layer.T_grad)
-        H_base = atleast_1d(self.layer.H_base)
-        T_base = atleast_1d(self.layer.T_base)
+        T_grad = np.atleast_1d(self.layer.T_grad)
+        H_base = np.atleast_1d(self.layer.H_base)
+        T_base = np.atleast_1d(self.layer.T_base)
         T = T_base + T_grad*(self.H - H_base)
         return T
 
@@ -455,7 +544,7 @@ class Atmosphere(DimensionalData):
         p = self.p
         T = self.T
         R_air = Phys.R_air
-        rho = p/(R_air*T)
+        rho = p/(R_air*T)  # TODO 2022-08-07: separate out
         return rho
 
     @_property_decorators
@@ -464,7 +553,7 @@ class Atmosphere(DimensionalData):
         T = self.T
         gamma_air = Phys.gamma_air
         R_air = Phys.R_air
-        a = sqrt(gamma_air*R_air*T)
+        a = np.sqrt(gamma_air*R_air*T)  # TODO 2022-08-07: separate out
         return a
 
     @_property_decorators
@@ -515,5 +604,5 @@ class Atmosphere(DimensionalData):
         sigma = Phys.collision_diam_air
 
         n = N_A*p/(R*T)  # number density
-        MFP = 1/(sqrt(2)*pi*sigma**2*n)
+        MFP = 1/(np.sqrt(2)*np.pi*sigma**2*n)
         return MFP
